@@ -51,6 +51,7 @@ from .writers.bibtex_writer import write_bibtex, write_bibtex_list
 from .writers.citation_writer import write_citation, write_citation_list
 from .writers.commonmeta_writer import write_commonmeta, write_commonmeta_list
 from .writers.crossref_xml_writer import (
+    CrossrefError,
     push_crossref_xml,
     push_crossref_xml_list,
     write_crossref_xml,
@@ -280,16 +281,6 @@ class Metadata:
 
     def write(self, to: str = "commonmeta", **kwargs) -> bytes | None:
         """convert metadata list into different formats"""
-        try:
-            return self._write_format(to, **kwargs)
-        except json.JSONDecodeError as e:
-            # More specific error message including the original JSONDecodeError details
-            raise ValueError(f"Invalid JSON: {str(e)}")
-        except Exception as e:
-            raise ValueError(f"Error writing format '{to}': {str(e)}")
-
-    def _write_format(self, to: str, **kwargs) -> bytes | None:
-        """Helper method to handle writing to different formats."""
         # JSON-based output formats
         if to in ["commonmeta", "datacite", "inveniordm", "schema_org", "csl"]:
             writer_map = {
@@ -321,9 +312,9 @@ class Metadata:
             self.email = kwargs.get("email", None)
             self.registrant = kwargs.get("registrant", None)
             output = write_crossref_xml(self)
-            self.write_errors = xml_schema_errors(output, schema="crossref_xml")
+            self.write_errors = xml_schema_errors(output, schema=to)
             if self.write_errors is not None:
-                self.is_valid = False
+                raise CrossrefError(self.write_errors)
             return output
         else:
             raise ValueError(f"Unsupported output format: {to}")
@@ -444,11 +435,16 @@ class MetadataList:
             else:
                 return output
         elif to == "crossref_xml":
-            output = write_crossref_xml_list(self)
-            if self.file and output is not None:
-                return write_output(self.file, output, [".xml"])
-            else:
-                return output
+            try:
+                output = write_crossref_xml_list(self)
+                if self.file and output is not None:
+                    return write_output(self.file, output, [".xml"])
+                else:
+                    return output
+            except (ValueError, CrossrefError) as e:
+                self.write_errors = str(e)
+                self.is_valid = False
+                return None
         elif to == "csl":
             output = json.dumps(write_csl_list(self))
             if self.file:
