@@ -1,7 +1,8 @@
 """crossref_xml reader for commonmeta-py"""
 
+from __future__ import annotations
+
 from collections import defaultdict
-from typing import Optional
 
 import requests
 
@@ -9,6 +10,7 @@ from ..author_utils import get_authors
 from ..base_utils import (
     compact,
     dig,
+    first,
     parse_attributes,
     parse_xml,
     presence,
@@ -45,10 +47,11 @@ def get_crossref_xml(pid: str, **kwargs) -> dict:
     if response.status_code != 200:
         return {"state": "not_found"}
 
-    return parse_xml(response.text, dialect="crossref") | {"via": "crossref_xml"}
+    dct = parse_xml(response.text, dialect="crossref")
+    return dct if isinstance(dct, dict) else {"state": "not_found"}
 
 
-def read_crossref_xml(data: dict, **kwargs) -> Commonmeta:
+def read_crossref_xml(data: dict | None, **kwargs) -> Commonmeta:
     """read_crossref_xml"""
     if data is None:
         return {"state": "not_found"}
@@ -160,7 +163,7 @@ def read_crossref_xml(data: dict, **kwargs) -> Commonmeta:
     if _type == "Article" and dig(publisher, "name") == "Front Matter":
         _type = "BlogPost"
 
-    url = parse_attributes(dig(bibmeta, "doi_data.resource"))
+    url = first(parse_attributes(dig(bibmeta, "doi_data.resource")))
     url = normalize_url(url)
     titles = crossref_titles(bibmeta)
     contributors = crossref_people(bibmeta)
@@ -239,13 +242,8 @@ def read_crossref_xml(data: dict, **kwargs) -> Commonmeta:
         crossref_reference(i) for i in wrap(dig(bibmeta, "citation_list.citation"))
     ]
     files = presence(meta.get("contentUrl", None))
-    provider = (
-        bibmeta.get("reg-agency").capitalize()
-        if bibmeta.get("reg-agency", None)
-        else None
-    )
-    if provider is None:
-        provider = get_doi_ra(_id)
+    provider = bibmeta.get("reg-agency", None)
+    provider = provider.capitalize() if provider else get_doi_ra(_id)
     state = "findable" if meta or read_options else "not_found"
 
     return {
@@ -283,15 +281,17 @@ def read_crossref_xml(data: dict, **kwargs) -> Commonmeta:
     } | read_options
 
 
-def crossref_titles(bibmeta):
+def crossref_titles(bibmeta: dict) -> list | None:
     """Title information from Crossref metadata."""
-    title = parse_attributes(dig(bibmeta, "titles.0.title"))
-    subtitle = parse_attributes(dig(bibmeta, "titles.0.subtitle"))
-    original_language_title = parse_attributes(
-        dig(bibmeta, "titles.0.original_language_title")
+    title = first(parse_attributes(dig(bibmeta, "titles.0.title")))
+    subtitle = first(parse_attributes(dig(bibmeta, "titles.0.subtitle")))
+    original_language_title = first(
+        parse_attributes(dig(bibmeta, "titles.0.original_language_title"))
     )
-    language = parse_attributes(
-        dig(bibmeta, "titles.0.original_language_title"), content="language"
+    language = first(
+        parse_attributes(
+            dig(bibmeta, "titles.0.original_language_title"), content="language"
+        )
     )
     if title is None and original_language_title is None:
         return None
@@ -316,10 +316,10 @@ def crossref_titles(bibmeta):
         ]
 
 
-def crossref_description(bibmeta):
+def crossref_description(bibmeta: dict) -> list:
     """Description information from Crossref metadata."""
 
-    def format_abstract(element):
+    def format_abstract(element: dict) -> dict:
         """Format abstract"""
         if isinstance(element.get("p", None), list):
             element["p"] = element["p"][0]
@@ -332,7 +332,7 @@ def crossref_description(bibmeta):
             {
                 "type": description_type,
                 "description": sanitize(
-                    parse_attributes(element, content="p", first=True)
+                    first(parse_attributes(element, content="p", first=True))
                 ),
             }
         )
@@ -340,7 +340,7 @@ def crossref_description(bibmeta):
     return [format_abstract(i) for i in wrap(bibmeta.get("abstract", None))]
 
 
-def crossref_people(bibmeta):
+def crossref_people(bibmeta: dict) -> list:
     """Person information from Crossref metadata."""
 
     person = dig(bibmeta, "contributors.person_name") or bibmeta.get(
@@ -387,11 +387,11 @@ def crossref_people(bibmeta):
     #           'name' => a['name'] || a['#text'] }
 
 
-def crossref_reference(reference: Optional[dict]) -> Optional[dict]:
+def crossref_reference(reference: dict | None) -> dict | None:
     """Get reference from Crossref reference"""
     if reference is None or not isinstance(reference, dict):
         return None
-    doi = parse_attributes(reference.get("doi", None))
+    doi = first(parse_attributes(reference.get("doi", None)))
     unstructured = reference.get("unstructured_citation", None)
     if isinstance(unstructured, dict):
         text = unstructured.get("font", None) or unstructured.get("#text", None)
@@ -490,12 +490,12 @@ def crossref_funding(funding: list) -> list:
     return []
 
 
-def crossref_license(licenses: list) -> dict:
+def crossref_license(licenses: list) -> dict | None:
     """Get license from Crossref"""
 
-    def map_element(element):
+    def map_element(element: dict) -> dict | None:
         """Format element"""
-        url = parse_attributes(element)
+        url = first(parse_attributes(element))
         url = normalize_cc_url(url)
         return dict_to_spdx({"url": url})
 
