@@ -1,6 +1,6 @@
 """InvenioRDM reader for Commonmeta"""
 
-from typing import Optional
+from __future__ import annotations
 
 import requests
 from furl import furl
@@ -13,7 +13,7 @@ from ..constants import (
     Commonmeta,
 )
 from ..date_utils import strip_milliseconds
-from ..doi_utils import doi_as_url, doi_from_url, is_rogue_scholar_doi
+from ..doi_utils import doi_as_url, doi_from_url, is_rogue_scholar_doi, validate_prefix
 from ..utils import (
     dict_to_fos,
     dict_to_spdx,
@@ -179,12 +179,17 @@ def read_inveniordm(data: dict, **kwargs) -> Commonmeta:
                 "type": "HasVersion",
             }
         )
-    elif meta.get("conceptdoi", None) or dig(meta, "links.parent_doi"):
+    elif meta.get("conceptdoi", None):
         relations.append(
             {
-                "id": doi_as_url(
-                    meta.get("conceptdoi") or dig(meta, "links.parent_doi")
-                ),
+                "id": doi_as_url(meta.get("conceptdoi")),
+                "type": "IsVersionOf",
+            }
+        )
+    elif validate_prefix(_id) == "10.59350" and dig(meta, "parent.id"):
+        relations.append(
+            {
+                "id": doi_as_url(f"10.59350/{dig(meta, 'parent.id')}"),
                 "type": "IsVersionOf",
             }
         )
@@ -235,7 +240,7 @@ def read_inveniordm(data: dict, **kwargs) -> Commonmeta:
 def get_references(references: list) -> list:
     """get_references"""
 
-    def get_reference(reference: dict) -> Optional[dict]:
+    def get_reference(reference: dict) -> dict | None:
         if not isinstance(reference, dict):
             return None
 
@@ -245,12 +250,10 @@ def get_references(references: list) -> list:
             id_ = normalize_url(reference.get("identifier"))
         else:
             id_ = reference.get("identifier")
-        return compact(
-            {
-                "id": id_,
-                "unstructured": reference.get("reference", None),
-            }
-        )
+        return {
+            "id": id_,
+            "unstructured": reference.get("reference", None),
+        }
 
     return [get_reference(i) for i in references]
 
@@ -258,7 +261,7 @@ def get_references(references: list) -> list:
 def get_citations(citations: list) -> list:
     """get citations."""
 
-    def get_citation(citation: dict) -> Optional[dict]:
+    def get_citation(citation: dict) -> dict | None:
         if not isinstance(citation, dict):
             return None
 
@@ -268,12 +271,10 @@ def get_citations(citations: list) -> list:
             id_ = normalize_url(citation.get("identifier"))
         else:
             id_ = citation.get("identifier")
-        return compact(
-            {
-                "id": id_,
-                "unstructured": citation.get("reference", None),
-            }
-        )
+        return {
+            "id": id_,
+            "unstructured": citation.get("reference", None),
+        }
 
     return [get_citation(i) for i in citations]
 
@@ -281,11 +282,11 @@ def get_citations(citations: list) -> list:
 def get_references_from_relations(references: list) -> list:
     """get_references_from_relations"""
 
-    def is_reference(reference):
+    def is_reference(reference) -> bool:
         """is_reference"""
         return reference.get("relationType", None) in ["Cites", "References"]
 
-    def map_reference(reference):
+    def map_reference(reference) -> dict:
         """map_reference"""
         identifier = reference.get("relatedIdentifier", None)
         identifier_type = reference.get("relatedIdentifierType", None)
@@ -335,7 +336,7 @@ def get_funding_references(funding_references: list) -> list:
     return [map_funding(i) for i in funding_references]
 
 
-def get_file(file: dict) -> str:
+def get_file(file: dict) -> dict | None:
     """get_file"""
     _type = file.get("type", None)
     return compact(
@@ -353,7 +354,7 @@ def get_file(file: dict) -> str:
 def get_relations(relations: list) -> list:
     """get_relations"""
 
-    def map_relation(relation: dict) -> Optional[dict]:
+    def map_relation(relation: dict) -> dict | None:
         """map_relation"""
         identifier = dig(relation, "identifier")
         scheme = dig(relation, "scheme")
@@ -388,7 +389,7 @@ def format_descriptions(descriptions: list) -> list:
     ]
 
 
-def format_identifier(identifier: dict) -> Optional[dict]:
+def format_identifier(identifier: dict) -> dict | None:
     """format_identifier. scheme url is stored as url metadata."""
     if (
         identifier.get("identifier", None) is None
