@@ -1,9 +1,16 @@
 """Commonmeta writer for commonmeta-py"""
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from ..base_utils import compact, omit
 
+if TYPE_CHECKING:
+    from ..metadata import Metadata, MetadataList
 
-def write_commonmeta(metadata):
+
+def write_commonmeta(metadata: Metadata | None) -> dict | None:
     """Write commonmeta"""
     if metadata is None:
         return None
@@ -36,13 +43,13 @@ def write_commonmeta(metadata):
     return compact(data)
 
 
-def write_commonmeta_list(metalist):
+def write_commonmeta_list(metalist: MetadataList | None) -> dict | None:
     """Write commonmeta list. If file is provided,
     write to file. Supports JSON, JSON Lines and YAML format."""
     if metalist is None:
         return None
 
-    def format_item(item):
+    def format_item(item) -> dict:
         """Format item for commonmeta list"""
         item = omit(vars(item), ["via", "is_valid"])
         return compact(item)
