@@ -21,6 +21,7 @@ from ..base_utils import (
     compact,
     dig,
     parse_xml,
+    presence,
     unparse_xml,
     unparse_xml_list,
     wrap,
@@ -106,7 +107,7 @@ class CrossrefXMLSchema(Schema):
 def convert_crossref_xml(metadata: Metadata) -> dict | None:
     """Convert Crossref XML"""
 
-    # return None if type is not supported by Crossref
+    # raise error if type is not supported by Crossref
     if metadata.type not in [
         "Article",
         "BlogPost",
@@ -121,13 +122,13 @@ def convert_crossref_xml(metadata: Metadata) -> dict | None:
         "Report",
         "Standard",
     ]:
-        log.error(f"Type not supported by Crossref: {metadata.id}")
-        return None
+        raise CrossrefError(
+            f"Type {metadata.type} not supported by Crossref: {metadata.id}"
+        )
 
-    # return None if doi or url are not present
+    # raise error if doi or url are not present
     if doi_from_url(metadata.id) is None or metadata.url is None:
-        log.error(f"DOI or URL missing for Crossref XML: {metadata.id}")
-        return None
+        raise CrossrefError(f"DOI or URL missing for Crossref XML: {metadata.id}")
 
     titles = get_titles(metadata)
     contributors = get_contributors(metadata)
@@ -362,25 +363,23 @@ def convert_crossref_xml(metadata: Metadata) -> dict | None:
             }
         )
     else:
-        log.error(f"Another error occured for Crossref XML: {metadata.id}")
-        data = None
+        raise CrossrefError(
+            f"Unexpected metadata type for Crossref XML: {metadata.type} for {metadata.id}"
+        )
+
     return data
 
 
 def write_crossref_xml(metadata: Metadata) -> bytes | None:
     """Write Crossref XML"""
-    if metadata is None or not metadata.is_valid:
-        log.error("Invalid metadata provided for Crossref XML generation")
-        return None
+    if metadata is None:
+        raise CrossrefError("No metadata provided for Crossref XML generation")
+
+    # Convert metadata to Crossref XML structure (raises CrossrefError on failure)
+    data = convert_crossref_xml(metadata)
 
     # Use the marshmallow schema to dump the data
     schema = CrossrefXMLSchema()
-
-    data = convert_crossref_xml(metadata)
-    if data is None:
-        log.error(f"Could not convert metadata to Crossref XML: {metadata.id}")
-        return None
-
     crossref_xml = schema.dump(data)
 
     # Ensure consistent field ordering through the defined mapping
@@ -394,14 +393,14 @@ def write_crossref_xml(metadata: Metadata) -> bytes | None:
     }
 
     # Convert to XML
-    return unparse_xml(crossref_xml, dialect="crossref", head=head)
+    xml_bytes = unparse_xml(crossref_xml, dialect="crossref", head=head)
+    return xml_bytes
 
 
 def write_crossref_xml_list(metalist: MetadataList) -> bytes | None:
     """Write crossref_xml list"""
     if metalist is None or not metalist.is_valid:
-        log.error("Invalid metalist provided for Crossref XML generation")
-        return None
+        raise CrossrefError("Invalid metalist provided for Crossref XML generation")
 
     # Use the marshmallow schema to dump the data
     schema = CrossrefXMLSchema()
@@ -418,12 +417,19 @@ def write_crossref_xml_list(metalist: MetadataList) -> bytes | None:
         field_order = [MARSHMALLOW_MAP.get(k, k) for k in list(data.keys())]
         crossref_xml = {k: crossref_xml[k] for k in field_order if k in crossref_xml}
         crossref_xml_list.append(crossref_xml)
+
+    # Raise error if there are write_errors
+    if presence(metalist.write_errors):
+        raise CrossrefError(f"Validation errors in metalist: {metalist.write_errors}")
+
     head = {
         "depositor": metalist.depositor,
         "email": metalist.email,
         "registrant": metalist.registrant,
     }
-    return unparse_xml_list(crossref_xml_list, dialect="crossref", head=head)
+
+    xml_bytes = unparse_xml_list(crossref_xml_list, dialect="crossref", head=head)
+    return xml_bytes
 
 
 def push_crossref_xml(
@@ -437,7 +443,12 @@ def push_crossref_xml(
 ) -> str:
     """Push crossref_xml to Crossref API, returns the API response."""
 
-    input_xml = write_crossref_xml(metadata)
+    try:
+        input_xml = write_crossref_xml(metadata)
+    except (ValueError, CrossrefError) as e:
+        log.error(f"Failed to generate XML for upload: {e}")
+        return "{}"
+
     if not input_xml:
         log.error("Failed to generate XML for upload")
         return "{}"
@@ -494,7 +505,12 @@ def push_crossref_xml_list(
 ) -> bytes | None:
     """Push crossref_xml list to Crossref API, returns the API response."""
 
-    input_xml = write_crossref_xml_list(metalist)
+    try:
+        input_xml = write_crossref_xml_list(metalist)
+    except ValueError as e:
+        log.error(f"Failed to generate XML for upload: {e}")
+        return None
+
     if not input_xml:
         log.error("Failed to generate XML for upload")
         return None
