@@ -1,6 +1,8 @@
 """DataCite writer for commonmeta-py"""
 
-from typing import Optional, Union
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
 from ..base_utils import compact, wrap
 from ..constants import (
@@ -10,15 +12,17 @@ from ..constants import (
     CM_TO_DC_TRANSLATIONS,
     CM_TO_RIS_TRANSLATIONS,
     CM_TO_SO_TRANSLATIONS,
-    Commonmeta,
 )
 from ..doi_utils import doi_from_url, normalize_doi
 
+if TYPE_CHECKING:
+    from ..metadata import Metadata, MetadataList
 
-def write_datacite(metadata: Commonmeta) -> Optional[Union[str, dict]]:
+
+def write_datacite(metadata: Metadata) -> dict | None:
     """Write datacite. Make sure JSON Schema validates before writing"""
     if metadata.write_errors is not None:
-        return "{}"
+        return {"errors": metadata.write_errors}
 
     alternate_identifiers = [
         {
@@ -73,17 +77,17 @@ def write_datacite(metadata: Commonmeta) -> Optional[Union[str, dict]]:
         else None
     )
 
-    def to_datacite_date(date: dict) -> dict:
-        """Convert dates to datacite dates"""
-        for k, v in date.items():
-            if k == "published":
-                k = "issued"
-            return {
-                "date": v,
-                "dateType": k.title(),
-            }
+    def to_datacite_date(date_item: tuple[str, str]) -> dict:
+        """Convert date tuple (key, value) to datacite date"""
+        k, v = date_item
+        if k == "published":
+            k = "issued"
+        return {
+            "date": v,
+            "dateType": k.title(),
+        }
 
-    dates = [to_datacite_date(i) for i in wrap(metadata.date)]
+    dates = [to_datacite_date(item) for item in metadata.date.items()]
 
     license_ = (
         [
@@ -146,10 +150,12 @@ def to_datacite_creator(creator: dict) -> dict:
         name = ", ".join([creator.get("familyName", ""), creator.get("givenName", "")])
     elif creator.get("name", None):
         name = creator.get("name", None)
+    else:
+        name = None
     name_identifiers = creator.get("id", None)
     if name_identifiers:
 
-        def format_name_identifier(name_identifier):
+        def format_name_identifier(name_identifier: str) -> dict:
             return {
                 "nameIdentifier": name_identifier,
                 "nameIdentifierScheme": "ORCID",
@@ -194,7 +200,7 @@ def to_datacite_related_identifier(reference: dict) -> dict:
     )
 
 
-def write_datacite_list(metalist):
+def write_datacite_list(metalist: MetadataList | None) -> list | None:
     """Write DataCite list"""
     if metalist is None:
         return None
