@@ -1,19 +1,30 @@
 """InvenioRDM writer for commonmeta-py"""
 
+from __future__ import annotations
+
 import logging
 from time import time
-from typing import Dict, Optional
+from typing import TYPE_CHECKING
 
 import orjson as json
 import requests
+from requests.exceptions import RequestException
 
-from ..base_utils import compact, dig, parse_attributes, presence, wrap
+from ..base_utils import (
+    compact,
+    dig,
+    first,
+    parse_attributes,
+    presence,
+    scrub,
+    unique,
+    wrap,
+)
 from ..constants import (
     CM_TO_INVENIORDM_TRANSLATIONS,
     COMMUNITY_TRANSLATIONS,
     CROSSREF_FUNDER_ID_TO_ROR_TRANSLATIONS,
     INVENIORDM_IDENTIFIER_TYPES,
-    Commonmeta,
 )
 from ..date_utils import get_iso8601_date
 from ..doi_utils import doi_from_url, is_rogue_scholar_doi, normalize_doi
@@ -28,13 +39,16 @@ from ..utils import (
     validate_ror,
 )
 
+if TYPE_CHECKING:
+    from ..metadata import Metadata, MetadataList
+
 log = logging.getLogger(__name__)
 
 
-def write_inveniordm(metadata):
+def write_inveniordm(metadata: Metadata) -> dict:
     """Write inveniordm"""
     if metadata is None or metadata.write_errors is not None:
-        return None
+        return {}
     _type = CM_TO_INVENIORDM_TRANSLATIONS.get(metadata.type, "Other")
     creators = [
         to_inveniordm_creator(i)
@@ -64,7 +78,7 @@ def write_inveniordm(metadata):
         for i in wrap(metadata.relations)
         if i.get("id", None) and i.get("type", None) != "IsPartOf"
     ]
-    funding = compact(
+    funding = unique(
         [
             to_inveniordm_funding(i)
             for i in wrap(metadata.funding_references)
@@ -120,8 +134,8 @@ def write_inveniordm(metadata):
                 {
                     "resource_type": {"id": _type},
                     "creators": creators,
-                    "title": parse_attributes(
-                        metadata.titles, content="title", first=True
+                    "title": first(
+                        parse_attributes(metadata.titles, content="title", first=True)
                     ),
                     "publisher": metadata.publisher.get("name", None)
                     if metadata.publisher
@@ -131,8 +145,10 @@ def write_inveniordm(metadata):
                     else None,
                     "dates": presence(dates),
                     "subjects": presence(subjects),
-                    "description": parse_attributes(
-                        metadata.descriptions, content="description", first=True
+                    "description": first(
+                        parse_attributes(
+                            metadata.descriptions, content="description", first=True
+                        )
                     ),
                     "rights": [{"id": metadata.license.get("id").lower()}]
                     if metadata.license.get("id", None)
@@ -206,12 +222,12 @@ def to_inveniordm_creator(creator: dict) -> dict:
     )
 
 
-def to_inveniordm_subject(sub: dict) -> Optional[dict]:
+def to_inveniordm_subject(sub: dict) -> dict | None:
     """Convert subject to inveniordm subject"""
     if sub.get("subject", None) is None:
         return None
-    if sub.get("subject").startswith("FOS: "):
-        subject = sub.get("subject")[5:]
+    elif str(sub.get("subject")).startswith("FOS: "):
+        subject = str(sub.get("subject"))[5:]
         id_ = FOS_MAPPINGS.get(subject, None)
         return compact(
             {
@@ -226,7 +242,7 @@ def to_inveniordm_subject(sub: dict) -> Optional[dict]:
     )
 
 
-def to_inveniordm_affiliations(creator: dict) -> Optional[list]:
+def to_inveniordm_affiliations(creator: dict) -> list | None:
     """Convert affiliations to inveniordm affiliations.
     Returns None if creator is not a person."""
 
@@ -241,12 +257,12 @@ def to_inveniordm_affiliations(creator: dict) -> Optional[list]:
     if creator.get("type", None) != "Person":
         return None
 
-    return compact(
+    return scrub(
         [format_affiliation(i) for i in wrap(creator.get("affiliations", None))]
     )
 
 
-def to_inveniordm_related_identifier(relation: dict) -> dict:
+def to_inveniordm_related_identifier(relation: dict) -> dict | None:
     """Convert relation to inveniordm related_identifier"""
     if normalize_doi(relation.get("id", None)):
         identifier = doi_from_url(relation.get("id", None))
@@ -258,24 +274,25 @@ def to_inveniordm_related_identifier(relation: dict) -> dict:
         return None
 
     # normalize relation types
-    relation_type = relation.get("type")
-    if relation.get("type") == "HasReview":
-        relation_type = "IsReviewedBy"
-    elif relation.get("type") == "IsPreprintOf":
-        relation_type = "IsPreviousVersionOf"
+    if relation.get("type", None) == "HasReview":
+        relation_type = "isreviewedby"
+    elif relation.get("type", None) == "IsPreprintOf":
+        relation_type = "ispreviousversionof"
+    elif relation.get("type", None) is not None:
+        relation_type = str(relation.get("type")).lower()
+    else:
+        return None
 
     return compact(
         {
             "identifier": identifier,
             "scheme": scheme,
-            "relation_type": {
-                "id": relation_type.lower(),
-            },
+            "relation_type": {"id": relation_type},
         }
     )
 
 
-def to_inveniordm_reference(reference: dict) -> dict:
+def to_inveniordm_reference(reference: dict) -> dict | None:
     """Convert reference to inveniordm reference"""
     if normalize_doi(reference.get("id", None)):
         identifier = doi_from_url(reference.get("id", None))
@@ -289,13 +306,14 @@ def to_inveniordm_reference(reference: dict) -> dict:
 
     if reference.get("unstructured", None) is None:
         # use title as unstructured reference
-        if reference.get("title", None):
-            unstructured = reference.get("title")
+        title = reference.get("title", None)
+        if title:
+            unstructured = str(title)
         else:
             unstructured = "Unknown title"
 
         if reference.get("publicationYear", None):
-            unstructured += " (" + reference.get("publicationYear") + ")."
+            unstructured += f" ({reference.get('publicationYear')})."
 
         return compact(
             {
@@ -305,7 +323,7 @@ def to_inveniordm_reference(reference: dict) -> dict:
             }
         )
     else:
-        unstructured = reference.get("unstructured")
+        unstructured = reference.get("unstructured", "")
 
         if reference.get("id", None):
             # remove duplicate ID from unstructured reference
@@ -323,7 +341,7 @@ def to_inveniordm_reference(reference: dict) -> dict:
         )
 
 
-def to_inveniordm_funding(funding: dict) -> Optional[dict]:
+def to_inveniordm_funding(funding: dict) -> dict | None:
     """Convert funding to inveniordm funding"""
     if funding.get("funderIdentifierType", None) == "Crossref Funder ID":
         # convert to ROR
@@ -383,13 +401,13 @@ def to_inveniordm_funding(funding: dict) -> Optional[dict]:
     )
 
 
-def write_inveniordm_list(metalist):
+def write_inveniordm_list(metalist: MetadataList) -> list | None:
     """Write InvenioRDM list"""
 
     if metalist is None:
         return None
 
-    def write_item(item):
+    def write_item(item) -> dict | None:
         """write inveniordm item for inveniordm list"""
 
         return write_inveniordm(item)
@@ -397,7 +415,7 @@ def write_inveniordm_list(metalist):
     return [write_item(item) for item in metalist.items]
 
 
-def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Dict:
+def push_inveniordm(metadata: Metadata, host: str, token: str, **kwargs) -> dict:
     """Push record to InvenioRDM"""
 
     try:
@@ -407,6 +425,7 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
 
         record = {
             "doi": doi,
+            "previous_doi": kwargs.get("previous_doi", None),
         }
 
         # extract optional information needed
@@ -414,7 +433,7 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
         # community_id is the id of the primary community of the record,
         # in the case of Rogue Scholar the blog community
 
-        if hasattr(metadata, "identifiers") and metadata.identifiers:
+        if metadata.identifiers:
             for identifier in metadata.identifiers:
                 if (
                     isinstance(identifier, dict)
@@ -424,7 +443,7 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
                     record["uuid"] = identifier.get("identifier")
                     continue
 
-        if hasattr(metadata, "relations") and metadata.relations:
+        if metadata.relations:
             community_index = None
             for i, relation in enumerate(metadata.relations):
                 if relation.get("type") == "IsPartOf" and relation.get(
@@ -449,9 +468,18 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
 
         # optionally update external services
         record = update_external_services(metadata, host, token, record, **kwargs)
-
+        return record
+    except ValueError as ve:
+        log.error(
+            f"Value error in push_inveniordm: {str(ve)}",
+            exc_info=True,
+            extra={"host": host, "record_id": metadata.id},
+        )
+        record = {
+            "doi": doi if "doi" in locals() else None,
+            "status": "error",
+        }
     except Exception as e:
-        print(f"Error in push_inveniordm: {str(e)}")
         log.error(
             f"Unexpected error in push_inveniordm: {str(e)}",
             exc_info=True,
@@ -462,7 +490,9 @@ def push_inveniordm(metadata: Commonmeta, host: str, token: str, **kwargs) -> Di
     return record
 
 
-def push_inveniordm_list(metalist, host: str, token: str, **kwargs) -> list:
+def push_inveniordm_list(
+    metalist: MetadataList, host: str, token: str, **kwargs
+) -> bytes | None:
     """Push inveniordm list to InvenioRDM, returns list of push results."""
 
     if metalist is None:
@@ -471,7 +501,12 @@ def push_inveniordm_list(metalist, host: str, token: str, **kwargs) -> list:
     return json.dumps(items, option=json.OPT_INDENT_2)
 
 
-def upsert_record(metadata: Commonmeta, host: str, token: str, record: dict) -> dict:
+def upsert_record(
+    metadata: Metadata,
+    host: str,
+    token: str,
+    record: dict,
+) -> dict:
     """Upsert InvenioRDM record, based on DOI"""
 
     output = write_inveniordm(metadata)
@@ -479,7 +514,19 @@ def upsert_record(metadata: Commonmeta, host: str, token: str, record: dict) ->
     # Check if record already exists in InvenioRDM
     record["id"] = search_by_doi(doi_from_url(record.get("doi")), host, token)
 
-    if record.get("id", None) is not None:
+    if record["previous_doi"] is not None:
+        record["previous_id"] = search_by_doi(
+            doi_from_url(record["previous_doi"]), host, token
+        )
+
+    if record.get("previous_id", None) is not None:
+        # Create a new version from the previous record
+        record["id"] = record["previous_id"]
+        record = create_new_version(record, host, token)
+
+        # Update new version
+        record = update_draft_record(record, host, token, output)
+    elif record.get("id", None) is not None:
         # Create draft record from published record
         record = edit_published_record(record, host, token)
 
@@ -497,7 +544,7 @@ def upsert_record(metadata: Commonmeta, host: str, token: str, record: dict) ->
 
 
 def add_record_to_communities(
-    metadata: Commonmeta, host: str, token: str, record: dict
+    metadata: Metadata, host: str, token: str, record: dict
 ) -> dict:
     """Add record to one or more InvenioRDM communities"""
 
@@ -514,7 +561,7 @@ def add_record_to_communities(
     # Add record to subject area community if subject area community is specified
     # Subject area communities should exist for all OECD subject areas
 
-    if hasattr(metadata, "subjects"):
+    if metadata.subjects:
         for subject in metadata.subjects:
             subject_name = subject.get("subject", "")
             slug = string_to_slug(subject_name)
@@ -525,12 +572,12 @@ def add_record_to_communities(
                 record = add_record_to_community(record, host, token, community_id)
 
     # Add record to communities defined as IsPartOf relation in InvenioRDM RelatedIdentifiers
-    if hasattr(metadata, "related_identifiers") and metadata.related_identifiers:
-        for identifier in metadata.related_identifiers:
-            if dig(identifier, "relation_type.id") == "ispartof" and identifier.get(
-                "identifier", ""
+    if metadata.relations:
+        for relation in metadata.relations:
+            if relation.get("type", None) == "IsPartOf" and relation.get(
+                "id", ""
             ).startswith(f"https://{host}/api/communities/"):
-                slug = identifier.get("identifier").split("/")[5]
+                slug = relation.get("id").split("/")[5]
                 community_id = search_by_slug(slug, "topic", host, token)
                 if community_id and community_id not in community_ids:
                     record = add_record_to_community(record, host, token, community_id)
@@ -539,20 +586,20 @@ def add_record_to_communities(
 
 
 def update_external_services(
-    metadata: Commonmeta, host: str, token: str, record: dict, **kwargs
+    metadata: Metadata, host: str, token: str, record: dict, **kwargs
 ) -> dict:
     """Update external services with changes in InvenioRDM"""
 
     # optionally update rogue-scholar legacy record
     if host == "rogue-scholar.org" and kwargs.get("legacy_key", None) is not None:
         record = update_legacy_record(
-            record, legacy_key=kwargs.get("legacy_key"), field="rid"
+            record, legacy_key=kwargs.get("legacy_key", None), field="rid"
         )
 
     return record
 
 
-def search_by_doi(doi, host, token) -> Optional[str]:
+def search_by_doi(doi, host, token) -> str | None:
     """Search for a record by DOI in InvenioRDM"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -568,12 +615,12 @@ def search_by_doi(doi, host, token) -> Optional[str]:
         if dig(data, "hits.total", 0) > 0:
             return dig(data, "hits.hits.0.id")
         return None
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error searching for DOI {doi}: {str(e)}", exc_info=True)
         return None
 
 
-def create_draft_record(record, host, token, output):
+def create_draft_record(record: dict, host: str, token: str, output: dict) -> dict:
     """Create a new draft record in InvenioRDM"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -598,13 +645,13 @@ def create_draft_record(record, host, token, output):
         record["updated"] = data.get("updated", None)
         record["status"] = "draft"
         return record
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error creating draft record: {str(e)}", exc_info=True)
         record["status"] = "error_draft"
         return record
 
 
-def reserve_doi(record, host, token) -> dict:
+def reserve_doi(record: dict, host: str, token: str) -> dict:
     """Reserve a DOI for a draft record."""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -620,7 +667,7 @@ def reserve_doi(record, host, token) -> dict:
         record["doi"] = data.get("doi", None)
         record["status"] = "doi_reserved"
         return record
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(
             f"Error reserving DOI for record {record['id']}: {str(e)}", exc_info=True
         )
@@ -628,7 +675,7 @@ def reserve_doi(record, host, token) -> dict:
         return record
 
 
-def edit_published_record(record, host, token):
+def edit_published_record(record: dict, host: str, token: str) -> dict:
     """Create a draft from a published record in InvenioRDM"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -640,11 +687,10 @@ def edit_published_record(record, host, token):
         )
         response.raise_for_status()
         data = response.json()
-        print("Edited published record:", dig(data, "parent.pids.doi.identifier"))
         record["updated"] = data.get("updated", None)
         record["status"] = "edited"
         return record
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(
             f"Error creating draft from published record: {str(e)}", exc_info=True
         )
@@ -652,7 +698,33 @@ def edit_published_record(record, host, token):
         return record
 
 
-def update_draft_record(record, host, token, inveniordm_data):
+def create_new_version(record: dict, host: str, token: str) -> dict:
+    """Create a new version of a published record in InvenioRDM"""
+    headers = {
+        "Authorization": f"Bearer {token}",
+        "Content-Type": "application/json",
+    }
+    try:
+        response = requests.post(
+            f"https://{host}/api/records/{record['id']}/versions", headers=headers
+        )
+        response.raise_for_status()
+        data = response.json()
+        record["id"] = data.get("id", None)
+        record["updated"] = data.get("updated", None)
+        record["status"] = "new_version"
+        return record
+    except RequestException as e:
+        log.error(
+            f"Error creating new version from published record: {str(e)}", exc_info=True
+        )
+        record["status"] = "error_create_new_version"
+        return record
+
+
+def update_draft_record(
+    record: dict, host: str, token: str, inveniordm_data: dict
+) -> dict:
     """Update a draft record in InvenioRDM"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -666,17 +738,16 @@ def update_draft_record(record, host, token, inveniordm_data):
         )
         response.raise_for_status()
         data = response.json()
-        print("Update draft record:", dig(data, "parent.pids.doi.identifier"))
         record["updated"] = data.get("updated", None)
         record["status"] = "updated"
         return record
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error updating draft record: {str(e)}", exc_info=True)
         record["status"] = "error_update_draft_record"
         return record
 
 
-def publish_draft_record(record, host, token):
+def publish_draft_record(record: dict, host: str, token: str) -> dict:
     """Publish a draft record in InvenioRDM"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -704,13 +775,13 @@ def publish_draft_record(record, host, token):
         record["updated"] = data.get("updated", None)
         record["status"] = "published"
         return record
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error publishing draft record: {str(e)}", exc_info=True)
         record["status"] = "error_publish_draft_record"
         return record
 
 
-def get_record_communities(record, host, token):
+def get_record_communities(record: dict, host: str, token: str) -> list | None:
     """Get record communities by id"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -726,12 +797,14 @@ def get_record_communities(record, host, token):
         if dig(data, "hits.total", 0) > 0:
             return dig(data, "hits.hits")
         return None
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error getting communities: {str(e)}", exc_info=True)
         return None
 
 
-def add_record_to_community(record, host, token, community_id):
+def add_record_to_community(
+    record: dict, host: str, token: str, community_id: str
+) -> dict:
     """Add a record to a community"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -746,12 +819,14 @@ def add_record_to_community(record, host, token, community_id):
         )
         response.raise_for_status()
         return record
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error adding record to community: {str(e)}", exc_info=True)
         return record
 
 
-def update_legacy_record(record, legacy_key: str, field: str = None) -> dict:
+def update_legacy_record(
+    record, legacy_key: str | None, field: str | None = None
+) -> dict:
     """Update corresponding record in Rogue Scholar legacy database."""
 
     legacy_host = "bosczcmeodcrajtcaddf.supabase.co"
@@ -795,18 +870,18 @@ def update_legacy_record(record, legacy_key: str, field: str = None) -> dict:
         }
         response = requests.patch(request_url, json=output, headers=headers, timeout=30)
         if response.status_code != 204:
-            return Exception(f"Unexpected status code: {response.status_code}")
+            raise Exception(f"Unexpected status code: {response.status_code}")
 
         record["status"] = "updated_legacy"
         return record
 
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error updating legacy record: {str(e)}", exc_info=True)
         record["status"] = "error_update_legacy_record"
         return record
 
 
-def search_by_slug(slug: str, type: str, host: str, token: str) -> Optional[str]:
+def search_by_slug(slug: str, type: str, host: str, token: str) -> str | None:
     """Search for a community by slug in InvenioRDM"""
     headers = {
         "Authorization": f"Bearer {token}",
@@ -822,7 +897,7 @@ def search_by_slug(slug: str, type: str, host: str, token: str) -> Optional[str]
         if dig(data, "hits.total", 0) > 0:
             return dig(data, "hits.hits.0.id")
         return None
-    except requests.exceptions.RequestException as e:
+    except RequestException as e:
         log.error(f"Error searching for community: {str(e)}", exc_info=True)
         return None
 
